REM Test the difference between linux clock and qb time after midnight.

REM This utility and source are public domain released on 01/15/2020.
REM   Release v1.1a adds timer loop.
REM   Release v1.2a adds /d and /t and /m to command line.
REM   Release v1.3a tries to parse strange characters in TIME$

REM Compile Vbdos:
REM   BC Testtime/x/v/o/g3;
REM   LINK Testtime,,,vbdos/e;

REM Doesn't work in QB45 becuase of lack of FORMAT$ support..
REM   And QB45 has no ON LOCAL ERROR support..

REM Remove remarks from following lines for BC 7.1 IDE:
REM   Then start with QBX /L LIBRARY.QLB

REM Declare Function FormatD$(byval a#, b$)
REM Declare Function Now#()
REM Declare Function Format$(x#, x$)
REM Function Format$(x#,x$) : Format$ = FormatD$(x#, x$) : End Function

'INT 1A - TIME - GET SYSTEM TIME
'        AH = 00h
'Return: CX:DX = number of clock ticks since midnight
'        AL = midnight flag, nonzero if midnight passed since time last read
'Notes:  There are approximately 18.2 clock ticks per second,
'         or 1800B0h per 24 hours.
'
'Notes:  Clock ticks are actually at 18.20648 per second single-precision.
'
'Notes:  Clock ticks are extended at 18.206481481481 per second calculated.

DEFINT A-Z
REM $DYNAMIC

TYPE RegTypeX
 AX AS INTEGER
 BX AS INTEGER
 CX AS INTEGER
 DX AS INTEGER
 BP AS INTEGER
 SI AS INTEGER
 DI AS INTEGER
 Flags AS INTEGER
 DS AS INTEGER
 ES AS INTEGER
END TYPE

DIM InregsX AS RegTypeX, OutregsX AS RegTypeX
DECLARE SUB InterruptX (N AS INTEGER, I AS RegTypeX, O AS RegTypeX)
ON ERROR GOTO ErrorRoutine

C$ = Command$

Var = Instr(C$,"/D")
Only.Display.Date=0
If Var Then
   Only.Display.Date=-1
   C$=Left$(C$,Var-1)+Mid$(C$,Var+2)
Endif

Var = Instr(C$,"/T")
Only.Display.Time=0
If Var Then
   Only.Display.Time=-1
   C$=Left$(C$,Var-1)+Mid$(C$,Var+2)
Endif

Var = Instr(C$,"/M")
Only.Display.Midnight=0
If Var Then
   Only.Display.Midnight=-1
   C$=Left$(C$,Var-1)+Mid$(C$,Var+2)
Endif

Var = Instr(C$,"/Z")
If Var Then
   debug=-1
   C$=Left$(C$,Var-1)+Mid$(C$,Var+2)
Endif

C$=Ltrim$(Rtrim$(C$))

If C$="" Then
   Gosub DisplayTime
   Color 7, 0
   End
Endif

If Left$(C$,2)="/R" Then
   C$ = Mid$(C$, 3)
   If Len(C$) Then
      R = Int(Val(C$))
      If R>=1 And R<=86400 Then
         E$=""
      Else
         R=1
      Endif
   Else
      R = 1
   Endif
   Gosub DisplayTime
   On Timer(R) Gosub DisplayTime
   Timer On
   Do
      If Inkey$<>"" Then
         Timer Off
         Color 7, 0
         End
      Endif
   Loop
Endif

color 15
print "Testtime utility v1.3a"
print "Usage:"
color 14
print "  Testtime [/r##][/d][/t][/m]"
color 15
print "Where:"
color 14
print "  /r  realtime display loop."
print "   ##  optional timer override in seconds."
print "  /d  only display date."
print "  /t  only display time."
print "  /m  only display midnight difference."
color 7
End

DisplayTime:
t2$ = TIME$ ' linux hour might not be 00-23

If debug Then ' experiment with time$
   Gosub Convert.Time ' returns h3!
Else
   ' convert ascii hour
   l = ASC(MID$(t2$, 1, 1)) - 48
   m = ASC(MID$(t2$, 2, 1)) - 48
   h3! = l * 10 + m
Endif

min! = CSNG(INT(VAL(MID$(t2$, 4, 2))))
sec! = CSNG(INT(VAL(MID$(t2$, 7, 2))))
t2! = h3! * 3600! + min! * 60! + sec!

t! = TIMER ' system timer (might be beyond 86,400)

' read timer from bios
InregsX.AX = &H0
CALL InterruptX(&H1A, InregsX, OutregsX)
Byte1! = CSNG(OutregsX.CX)
Byte2! = CSNG(OutregsX.DX)
IF Byte1! < 0! THEN Byte1! = Byte1! + 65536! ' adjust signed-integer
IF Byte2! < 0! THEN Byte2! = Byte2! + 65536!
Byte! = Byte1! * 65536! + Byte2!

color 15
print "Realtime clock analyzer:";
if debug then print " (debug)";
print
PRINT "------------------------"

' calculate midnight only
If Only.Display.Midnight Then
   Goto Midnight
Endif

' actual precision: 18.206481481481#
x$ = Str$(&H1800B0 / 86400!) ' per 24 hours
x$ = Left$(x$, len(x$) - 1)
If Only.Display.Time=0 Then
   If Only.Display.Date=0 Then
      PRINT "Ticks per second="
      color 14
      PRINT x$; " (calculated)"
      PRINT Byte! / t!; "(realtime)"
   Endif
Endif

Byte! = Byte! / 18.20648!

color 15
If Only.Display.Time=0 Then
   PRINT "clock date: "; DATE$ ' linux date
   PRINT "qb format$ clock: "; Format$(Now,"mm-dd-yyyy") ' dos date
Endif
If Only.Display.Date=0 Then
   PRINT "clock time: "; TIME$
   color 14
   PRINT " BIOS:  "; Byte!
   PRINT " QB:    "; t!
   PRINT " TIME$: "; t2!
   color 15
   t1$ = Format$(Now, "hh:mm:ss") ' get dos time
   t1! = CSNG(INT(VAL(MID$(t1$, 1, 2)))) * 3600! + CSNG(INT(VAL(MID$(t1$, 4, 2)))) * 60! + CSNG(INT(VAL(MID$(t1$, 7, 2))))
   PRINT "qb format$ clock: ";t1$; " seconds: ";Format$(CDBL(t1!), "#,##0;;0")
Endif
color 15
If Only.Display.Date=0 And Only.Display.Time=0 Then
Midnight:
   t1$ = Format$(Now, "hh:mm:ss") ' get dos time
   t1! = CSNG(INT(VAL(MID$(t1$, 1, 2)))) * 3600! + CSNG(INT(VAL(MID$(t1$, 4, 2)))) * 60! + CSNG(INT(VAL(MID$(t1$, 7, 2))))
   If Only.Display.Midnight Then
      PRINT "qb format$ clock: ";t1$; " seconds: ";Format$(CDBL(t1!), "#,##0;;0")
   Endif
   T#=24#*60#*60# ' calculate seconds in a day
   PRINT "midnight: "; Format$(T#,"#,##0;;0") ;" seconds."
   PRINT "midnight difference: ";Format$(CDBL(86400!-t1!), "#,##0;;0");" seconds."
Endif
If Only.Display.Midnight Then
   Return
Endif
If Only.Display.Date And Only.Display.Time Then
   print "No information available."
Endif
Return
END

ErrorRoutine:
 Print "Error";Err
 End

Convert.Time:
REM time$ = :9:59:59
REM note: ascii hours may be 255^2 or 65025 which would be 7.4 years
REM theory: 3 digit hours would be 255^3 or 1,892.8 years..

s$ = TIME$

h$ = LEFT$(s$, LEN(s$) - 3)
h$ = LEFT$(h$, LEN(h$) - 3)

s! = VAL(RIGHT$(s$, 2)) ' seconds
m$ = LEFT$(s$, LEN(s$) - 3)
m! = VAL(RIGHT$(m$, 2)) ' minutes

' hours in ascii (might be 3 char!?)
h3! = 0!
FOR x = 1 TO LEN(h$)
    l = ASC(MID$(h$, x, 1)) - 48
    h3! = h3! * 10 + l
NEXT

' seconds past last boot
x! = h3! * 3600 + m! * 60 + s!
RETURN
